// Basic Example for Controlling Dissolved Oxygen with Open Source Bio
// A Switching Relay is connected to an air pump
// The Relay is connected to pin 12 and Ground
// The commands below are necessary for the OSB library to work
#include <Wire.h>
#include <Boekel_OpenSourceBio.h>
using namespace Boekel;
OpenSourceBio osb; // create osb object.

// Creating a variable: Pin 12 on the arduino board will be used to control the pump, we will call it pump
// Connect pin 12 to the positive signal side of the relay and the ground to the negative side
const int pump = 12; 
// Now we can write "pump" in our code instead of "12"

// The setup() function will only run once. 
void setup()
{
  delay(1500);  // The main board takes a second to warm up.  This delay ensures it is ready to communicate
  osb.clearScreen(OpenSourceBio::COLOR_BLACK); // clears the user programmable screen 
  pinMode(pump, OUTPUT); //Setting pin 12 "pump" as an output 
}


// The loop() function will run for ever 
void loop()
{

  osb.clearScreen(OpenSourceBio::COLOR_BLACK); // clears user programmable screen
  
  //Creating a varaiable: we will store the "reading" as a double varaible
  double reading;
  
  //The OSB library command below updates the readings from the Transmitter
  osb.updateReadings(); 
  
  //We are setting the reading varaible equal to DO value with the osb.getDOmgl(); command in the OSB library  
  reading = osb.getDOmgl();  
   
  //The "if" loop will run if the DO is below 6.5mg/L 
   if (reading <= 6.5)  
           {
            // Displays text on screen(x,y, text color, background color)
            osb.displayText(100, 100, OpenSourceBio::COLOR_RED, OpenSourceBio::COLOR_BLACK,"Adding Air");
            
            
            digitalWrite(pump, HIGH); // Turns air pump relay on 
            delay(1500); //Keep pump on for 1.5 seconds
            digitalWrite(pump, LOW); //Turn pump off
            delay(500); //Pause for 0.5 seconds    
            
            }
  else // The "else" displays text if the reading is OK
           {
             osb.displayText(60, 100,OpenSourceBio::COLOR_GREEN, OpenSourceBio::COLOR_BLACK,"Dissolved Oxygen OK");
             delay(2000); //Delays for 2 seconds
             
           } 
  
  
}
